//------------------------------------------------------
// Compute the equilibrium vector of the maze problem, 
// where a mouse in a 3x3 grid maze to neighboring sites 
// with equal probability(see sketch).
//
// |-----|-----|-----|
// |  1  |  2  |  3  |
// |-   -|-   -|-   -|
// |  4     5     6  |
// |-   -|-   -|-   -|
// |  7  |  8  |  9  |
// |-----|-----|-----|
//
//------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/uniform_real_distribution.hpp>

#define JK 1000
boost::random::mt19937 gen;

void jk(double &avg, double &err, double* jk, int N){
  avg = 0.0;
  double tmp = 0.0;
  double jk_sum = 0.0;
  for(int i=0; i<N; i++){
    jk_sum += jk[i];
  }
  avg = jk_sum/(double)N;

  for(int i=0; i<N; i++){
    tmp += pow(avg - jk[i], 2);
  }
  err = tmp * (double)(N-1) / (double)N ;
  err = sqrt(err);
}

void Tmatrix_MulEqul(double** M2, double **M, int N){
  double **T1 = new double*[N];
  double **T2 = new double*[N];
  for(int i = 0 ; i < N ; i++){
    T1[i] = new double[N];
    T2[i] = new double[N];
    for(int j = 0 ; j < N ; j++){
      T1[i][j] = 0.0;
      T2[i][j] = 0.0;
    }
  }

  //T1=M2
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      T1[i][j] = M2[i][j];
    }
  }

  //T2=0
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      T2[i][j] = 0.0;
    }
  }

  //T2=T1*M
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      for(int k = 0 ; k < N ; k++){
        T2[i][j] += T1[i][k]*M[k][j];
      }
    }
  }

  //M=T1
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      M2[i][j] = T2[i][j];
    }
  }

  for(int i = 0 ; i < N ; i++){
    delete[] T1[i];
    delete[] T2[i];
  }
  delete[] T1;
  delete[] T2;
}

int irand(int N){
  int ran = rand()%N;
  return ran;
}

double drand(){
  boost::random::uniform_real_distribution<> dist(0, 1);
  return dist(gen);
}


int main(int argc, char* argv[]){

  if(argc != 2){
    printf("input n\n");
    exit(0);
  }

  int N = 9; // the number of balls
  int n = atoi(argv[1]);
  //------------------------
  // analytic calculation
  // using transition matrix
  //------------------------
  double **M = new double*[N];
  double **M2 = new double*[N];
  for(int i = 0 ; i < N ; i++){
    M[i] = new double[N];
    M2[i] = new double[N];
    for(int j = 0 ; j < N ; j++){
      M[i][j] = 0.0;
      M2[i][j] = 0.0;
    }
  }

  //--------------------------
  // set the transition matrix
  //--------------------------
  M[0][3]=1.0;
  M[1][4]=1.0;
  M[2][5]=1.0;
  M[3][0]=M[3][4]=M[3][6]=1.0/3.0; 
  M[4][1]=M[4][3]=M[4][5]=M[4][7]=1.0/4.0;
  M[5][2]=M[5][4]=M[5][8]=1.0/3.0;
  M[6][3]=1.0;
  M[7][4]=1.0;
  M[8][5]=1.0;

  //-------------------
  // equilibrium vector
  //-------------------
  double W[9] = {1.0, 1.0, 1.0, 3.0, 4.0, 3.0, 1.0, 1.0, 1.0};
  double W2[9] = {0.0};
  for(int i = 0 ; i < N ; i++){
  W[i] /= 16.0;
  }

  //----------------------------
  //print the equilibrium vector
  //----------------------------
  printf("W = ");
  for(int i = 0 ; i < N ; i++){
    printf("%.4lf ",W[i]);
  }
  printf("\n");
  printf("\n");


  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      M2[i][j] = M[i][j];
    }
  }

  //----------------
  // calculate P^(n)
  //----------------
  for(int i = 0 ; i < n-1 ; i++){
    Tmatrix_MulEqul(M2, M, N);
  }

  //----------------------
  //print the P^(n) matrix
  //----------------------
  printf("P^(%d) = \n", n);
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      printf("%.3lf ",M2[i][j]);
    }
    printf("\n");
  }
  printf("\n");

  //---------------------
  // Multiply W and P^(n)
  //---------------------
  for(int i = 0 ; i < N ; i++){
    for(int j = 0 ; j < N ; j++){
      W2[j]+= W[i] * M2[i][j];
    }
  }
  
  //------------------------------
  // print the result of W * P^(n)
  //------------------------------
  printf("W*P^(%d) = ", n);
  for(int i = 0 ; i < N ; i++){
    printf("%.4lf ",W2[i]);
  }
  printf("\n");
  printf("\n");


  // save the result as file
  FILE *fp_anal;
  char fname_anal[1024];
  sprintf(fname_anal,"anal_%d.dat", n);
  fp_anal = fopen(fname_anal, "w");

  double *V = new double[N];
  // P^(n)
  for(int i = 0 ; i < N ; i++){
    if(M2[0][i] > 1.0e-10){
      V[i] = M2[0][i];
    }
  }

  // P^(n+1)
  Tmatrix_MulEqul(M2, M, N);
  for(int i = 0 ; i < N ; i++){
    if(M2[0][i] > 1.0e-10){
      V[i] = M2[0][i];
    }
  }
  
  for(int i = 0 ; i < N ; i++){
    fprintf(fp_anal, "%d %lf\n", i+1, V[i]);
  }

  //-----
  // MCMC
  //-----

  int **Z = new int*[N];
  for(int i = 0 ; i < N ; i++){
    Z[i] = new int[JK] ;
    for(int j = 0 ; j < JK ; j++){
      Z[i][j] = 0;
    }
  }

  // n steps
  for(int i_jk = 0 ; i_jk < JK ; i_jk++){
      int x = 0 ; //starting point
      for(int i = 0 ; i < n ; i++){
        double P[9] = {0.0};
        for(int j = 0 ; j < N ; j++){
          for(int k = 0 ; k < j+1 ; k++){
            P[j] += M[x][k];
          }
        }
        double P_move = drand();
        for(int j = 0 ; j < N ; j++){
          if(P_move < P[j]){
            x=j;
            break;
          }
        }
      }
      Z[x][i_jk]++;
  }
  
  // n+1 steps
  for(int i_jk = 0 ; i_jk < JK ; i_jk++){
      int x = 0 ; //starting point
      for(int i = 0 ; i < n+1 ; i++){
        double P[9] = {0.0};
        for(int j = 0 ; j < N ; j++){
          for(int k = 0 ; k < j+1 ; k++){
            P[j] += M[x][k];
          }
        }
        double P_move = drand();
        for(int j = 0 ; j < N ; j++){
          if(P_move < P[j]){
            x=j;
            break;
          }
        }
      }
      Z[x][i_jk]++;
  }


  double **jk_avg = new double*[N];
  for(int i = 0 ; i < N ; i++){
    jk_avg[i] = new double[JK];
    for(int j = 0 ; j < JK ; j++){
      jk_avg[i][j] = 0.0;
    }
  }
  
  for(int i = 0 ; i < N ; i++){
    for(int del = 0 ; del < JK ; del++){
      for(int j = 0 ; j < JK ; j++){
        if(j == del) continue;
        jk_avg[i][del] += (double)Z[i][j];
      }
      jk_avg[i][del] /= (double)(JK-1);
    }
  }
  double *avg = new double[N];
  double *err = new double[N];
  for(int i = 0 ; i < N ; i++){
    jk(avg[i], err[i], jk_avg[i], JK);
  }

  // save the result as file
  FILE *fp_dat;
  char fname_dat[1024];
  sprintf(fname_dat,"data_%d.dat", n);
  fp_dat = fopen(fname_dat, "w");

  for(int i = 0 ; i < N ; i++){
    fprintf(stdout, "%d %lf %lf\n", i+1, avg[i], err[i]);
    fprintf(fp_dat, "%d %lf %lf\n", i+1, avg[i], err[i]);
  }

  for(int i = 0 ; i < N ; i++){
    delete[] M[i];
    delete[] M2[i];
    delete[] jk_avg[i];
  }
  delete[] V;
  delete[] M;
  delete[] M2;
  delete[] Z;
  delete[] jk_avg;
  delete[] avg;
  delete[] err;



  return 0;
}
